/*
 * This file is part of the Extra-P software (http://www.scalasca.org/software/extra-p)
 *
 * Copyright (c) 2017,
 * Technische Universitaet Darmstadt, Germany
 *
 * This software may be modified and distributed under the terms of
 * a BSD-style license.  See the COPYING file in the package base
 * directory for details.
 *
 */

#ifndef EXTRAP_MULTIPARAMETERFUNCTION_HPP
#define EXTRAP_MULTIPARAMETERFUNCTION_HPP

#include <EXTRAP_Function.hpp>
#include <EXTRAP_MultiParameterTerm.hpp>

namespace EXTRAP
{
class MultiParameterFunction : public Function
{
public:
    static const std::string MULTIPARAMETERFUNCTION_PREFIX;
    MultiParameterFunction( void );
    MultiParameterFunction( const MultiParameterFunction& obj );

    virtual Value
    evaluate( const ParameterValueList& parameterValues ) const;

    virtual std::string
    getAsString( const ParameterList& parameterNames ) const;

    void
    addMultiParameterTerm( const MultiParameterTerm& ct );

    std::vector<MultiParameterTerm>&
    getMultiParameterTerms( void );

    const std::vector<MultiParameterTerm>&
    getMultiParameterTerms( void ) const;

    void
    setConstantCoefficient( Value v );

    Value
    getConstantCoefficient( void ) const;

    virtual Function*
    clone( void ) const;

    virtual bool
    serialize( IoHelper* ioHelper ) const;

    static MultiParameterFunction*
    deserialize( IoHelper* ioHelper );

private:
    std::vector<MultiParameterTerm> m_multi_parameter_terms;
    Value                           m_constant_coefficient;
};
bool
equal( const MultiParameterFunction* lhs,
       const MultiParameterFunction* rhs );
};

#endif
