## -*- mode: autoconf -*-

##
## This file is part of the Extra-P software (http://www.scalasca.org/software/extra-p)
##
## Copyright (c) 2016-2017,
## Technische Universitaet Darmstadt, Germany
##
## This software may be modified and distributed under the terms of
## a BSD-style license.  See the COPYING file in the package base
## directory for details.
##

## file       extrap_cube.m4

AC_DEFUN([EXTRAP_CUBE43], [

# Check Cube support
AC_ARG_WITH([cube], AS_HELP_STRING([--with-cube=DIR],[Path to Cube installation]),
    [AC_PATH_PROGS([cube_config],[cube-config],[],[$withval/bin])],
    [AC_PATH_PROGS([cube_config],[cube-config],[])])

# Bail out if 'cube_config' is not set
if test "x${cube_config}" == "x"
then
    AC_MSG_ERROR([Error, cube-config version 4.3 and higher is required])
fi

# Bail out if 'cube_config' does not support the required parameters
if test "x`${cube_config} --cube-ldflags`" == "x"
then
    AC_MSG_ERROR([Error, Incompatible cube-config found! You need Cube version 4.3 and higher to build.])
fi

# Setting Cube paths for lib
cube_cxxflags="`${cube_config} --cube-cxxflags`"
AC_SUBST([cube_cxxflags])
cube_include_path="`${cube_config} --cube-include-path`"
AC_SUBST([cube_include_path])
cube_ldflags="`${cube_config} --cube-ldflags`"
AC_SUBST([cube_ldflags])
cube_lib="`${cube_config} --cube-library`"
AC_SUBST([cube_lib])
]);


AC_DEFUN([EXTRAP_CUBE], [

# Check Cube support
AC_ARG_WITH([cube], AS_HELP_STRING([--with-cube=DIR],[Path to Cube installation]),
    [AC_PATH_PROGS([cube_config],[cubelib-config],[],[$withval/bin])],
    [AC_PATH_PROGS([cube_config],[cubelib-config],[])])

# Bail out if 'cube_config' is not set
if test "x${cube_config}" == "x"
then
    EXTRAP_CUBE43
else

    # Bail out if 'cube_config' does not support the required parameters
    if test "x`${cube_config} --ldflags`" == "x"
    then
        AC_MSG_ERROR([Error, incompatible cubelib-config found! You need cubelib version 4.4 and higher to build.])
    fi

    # Setting Cube paths for lib
    cube_cxxflags="`${cube_config} --cppflags`"
    AC_SUBST([cube_cxxflags])
    cube_include_path="`${cube_config} --include`"
    AC_SUBST([cube_include_path])
    cube_ldflags="`${cube_config} --ldflags`"
    AC_SUBST([cube_ldflags])
    cube_lib="`${cube_config} --libs`"
    AC_SUBST([cube_lib])
fi

AC_MSG_CHECKING([Cube library])
AC_LANG_PUSH(C++)
save_CPPFLAGS="$CPPFLAGS"
CPPFLAGS="$CPPFLAGS $cube_cxxflags"
save_LIBS="$LIBS"
LIBS="$LIBS $cube_ldflags $cube_lib"
AC_LINK_IFELSE([
     AC_LANG_SOURCE([
#include <Cube.h>
int main()
{
    cube::Cube c;
    c.openCubeReport("test");    
}
])],[
    AC_MSG_RESULT([yes])
],[
    AC_MSG_RESULT([no])
    AC_MSG_ERROR([Error, Failed to link against Cube library. Either no Cube library found or not compatible with the C++ compiler. Cube should be build with the same C++ compiler.])
])
CPPFLAGS="$save_CPPFLAGS"
LIBS="$save_LIBS"

AC_LANG_POP(C++)
]);
