/*
 * This file is part of the Extra-P software (http://www.scalasca.org/software/extra-p)
 *
 * Copyright (c) 2017,
 * Technische Universitaet Darmstadt, Germany
 *
 * This software may be modified and distributed under the terms of
 * a BSD-style license.  See the COPYING file in the package base
 * directory for details.
 *
 */

#include <config.h>
#include <EXTRAP_Coordinate.hpp>
#include <EXTRAP_Utilities.hpp>
#include <EXTRAP_Experiment.hpp>

namespace EXTRAP
{
const std::string Coordinate::COORDINATE_PREFIX = "Coordinate";

int64_t
Coordinate::getId( void ) const
{
    return m_id;
}

void
Coordinate::setId( int64_t id )
{
    m_id = id;
}

bool
Coordinate::serialize( IoHelper* ioHelper )
{
    SAFE_RETURN( ioHelper->writeString(  Coordinate::COORDINATE_PREFIX ) );
    SAFE_RETURN( ioHelper->writeId(  getId() ) );
    SAFE_RETURN( ioHelper->writeInt(  size() ) );

    //write ParameterValues
    for ( Coordinate::iterator it = begin();
          it != end();
          it++ )
    {
        Parameter p = it->first;
        SAFE_RETURN( ioHelper->writeString(  p.getName() ) );
        Value val = it->second;
        SAFE_RETURN( ioHelper->writeValue(  val ) );
    }
    return true;
}

Coordinate*
Coordinate::deserialize(                          const Experiment* experiment, IoHelper* ioHelper )
{
    int64_t id     = ioHelper->readId();
    int     length = ioHelper->readInt();

    DebugStream << "Read Coordinate:\n"
                << " coordinate id:  " << id << "\n"
                << " num parameters: " << length << std::endl;

    Coordinate* coordinate = new Coordinate();
    for ( int i = 0; i < length; i++ )
    {
        Parameter param( ioHelper->readString() );
        Value     val = ioHelper->readValue();
        coordinate->insert( std::pair<Parameter, Value>( param, val ) );
    }
    return coordinate;
}
};
