/*
 * This file is part of the Extra-P software (http://www.scalasca.org/software/extra-p)
 *
 * Copyright (c) 2017,
 * Technische Universitaet Darmstadt, Germany
 *
 * This software may be modified and distributed under the terms of
 * a BSD-style license.  See the COPYING file in the package base
 * directory for details.
 *
 */
#include <config.h>
#include <EXTRAP_MultiParameterSimpleFunctionModeler.hpp>
#include <EXTRAP_SingleParameterHypothesis.hpp>
#include <EXTRAP_Interface.hpp>
#include <EXTRAP_Interface.hpp>

#include <sstream>

namespace EXTRAP
{
const std::string BAD_MODEL               = "This model has a large error metric (SMAPE). Probably the measurements can not be explained by PMNF.";
const std::string BETTER_USER_EXPECTATION = "Expectation function specified by user is a better model than the best hypothesis.";


MultiParameterHypothesis
MultiParameterSimpleFunctionModeler::createModel( const std::vector<DataPoint>& modeledDataPointList,
                                                  ModelCommentList&             comments,
                                                  const Function*               expectationFunction )
{
    double constantCost = 0;



    double meanModel = 0;

    for ( int i = 0; i < modeledDataPointList.size(); i++ )
    {
        meanModel += modeledDataPointList[ i ].getValue() / ( double )modeledDataPointList.size();
    }
    for ( int i = 0; i < modeledDataPointList.size(); i++ )
    {
        constantCost += ( modeledDataPointList[ i ].getValue() - meanModel ) * ( modeledDataPointList[ i ].getValue() - meanModel );
    }

    //EXTRAP::MultiParameterFunction constantMP=this->createConstantModel(modeledDataPointList);
    //constantCost
    //First the data points have to be split by parameter

    std::vector<std::vector<DataPoint> > singleParamDataPointLists;
    int                                  parameterNumbers;
    std::vector<EXTRAP::Parameter>       params, tmpparams;
    assert( modeledDataPointList.size() > 0 );
    parameterNumbers = modeledDataPointList[ 0 ].getParameterList().size();
    for ( int i = 1; i < modeledDataPointList.size(); i++ )
    {
        assert( parameterNumbers == modeledDataPointList[ i ].getParameterList().size() );
    }
    for ( EXTRAP::ParameterValueList::const_iterator it = modeledDataPointList[ 0 ].getParameterList().begin(); it != modeledDataPointList[ 0 ].getParameterList().end(); it++ )
    {
        const Parameter p = it->first;
        params.push_back( p );
        tmpparams.push_back( p );
        std::vector<DataPoint> singleParamDataPointList;
        std::vector<bool>      valueUsed( modeledDataPointList.size(), false );
        for ( int j = 0; j < modeledDataPointList.size(); j++ )
        {
            if ( valueUsed[ j ] == true )
            {
                continue;
            }
            Value v           = 0;
            int   sampleCount = 0;
            for ( int k = j; k < modeledDataPointList.size(); k++ )
            {
                if ( modeledDataPointList[ j ].getParameterValue( p ) == modeledDataPointList[ k ].getParameterValue( p ) )
                {
                    v += modeledDataPointList[ j ].getValue();
                    sampleCount++;
                    valueUsed[ k ] = true;
                    //std::cout<<k<<" "<<p.getName()<<" "<<modeledDataPointList[k].getParameterValue(p)<<" "<<modeledDataPointList[k].getValue()<<std::endl;
                }
            }
            v = v / ( double )sampleCount;
            ParameterValueList* temporaryPVL;
            temporaryPVL = new ParameterValueList();
            Interval interval;
            interval.end   = 0;
            interval.start = 0;
            temporaryPVL->insert( std::make_pair( p, modeledDataPointList[ j ].getParameterValue( p ) ) );
            //std::cout<<v<<" "<<sampleCount<<" "<<modeledDataPointList[j].getParameterValue(p)<<" "<<p.getName()<<std::endl;
            const DataPoint t( temporaryPVL, sampleCount, v, interval );
            singleParamDataPointList.push_back( t );
        }
        singleParamDataPointLists.push_back( singleParamDataPointList );
    }


    std::vector<EXTRAP::SingleParameterHypothesis> bestSingleParameterHypotheses;
    std::vector<int>                               paramsToDelete;
    int                                            paramToKeep;
    for ( int i = 0; i < singleParamDataPointLists.size(); i++ )
    {
        EXTRAP::SingleParameterHypothesis hyp = this->m_single_parameter_function_modeler->createModel( singleParamDataPointLists[ i ], comments, expectationFunction );
        if ( hyp.getFunction()->getCompoundTerms().size() > 0 )
        {
            bestSingleParameterHypotheses.push_back( hyp );
            paramToKeep = i;
        }
        else
        {
            paramsToDelete.push_back( i );
        }

        //std::cout << i << " " << hyp.getFunction()->getAsString( "p" ) << std::endl;
    }



    if ( paramsToDelete.size() == 1 )
    {
        MultiParameterFunction* simpleFunction = new MultiParameterFunction();
        MultiParameterTerm      t;
        // std::cout<<"One Parameter"<<std::endl;
        t.addCompoundTermParameterPair( bestSingleParameterHypotheses[ 0 ].getFunction()->getCompoundTerms()[ 0 ], params[ paramToKeep ] );
        t.setCoefficient( bestSingleParameterHypotheses[ 0 ].getFunction()->getCompoundTerms()[ 0 ].getCoefficient() );
        simpleFunction->addMultiParameterTerm( t );
        MultiParameterHypothesis simpleHypothesis( simpleFunction );
        simpleHypothesis.getFunction()->setConstantCoefficient( bestSingleParameterHypotheses[ 0 ].getFunction()->getConstantCoefficient() );

        simpleHypothesis.computeCost( modeledDataPointList );
        //std::cout << "Output " << simpleHypothesis.getFunction()->getAsString( params ) << std::endl;
        return simpleHypothesis;
    }

    if ( paramsToDelete.size() == 2 )
    {
        //  std::cout<<"Constant"<<std::endl;
        MultiParameterFunction* constantFunction = new MultiParameterFunction();
        constantFunction->setConstantCoefficient( meanModel );
        //    MultiParameterTerm t;
        //    constantFunction->addMultiParameterTerm(t);
        MultiParameterHypothesis constantHypothesis( constantFunction );
        //constantHypothesis.computeCost(modeledDataPointList);
        constantHypothesis.setRSS( constantCost );
        constantHypothesis.setAR2( 0 );
        constantHypothesis.setrRSS( 0 );
        constantHypothesis.setSMAPE( 0 );
        //std::cout<<"Output "<<constantHypothesis.getFunction()->getAsString(tmpparams)<<std::endl;
        //std::cout<<"Const coeff: "<< constantHypothesis.getFunction()->getConstantCoefficient()<<std::endl;
        return constantHypothesis;
    }
    for ( int i = paramsToDelete.size() - 1; i >= 0; i-- )
    {
        //std::cout<<"deleting "<<paramsToDelete[i]<<std::endl;
        bestSingleParameterHypotheses.erase( bestSingleParameterHypotheses.begin() + paramsToDelete[ i ] );
        params.erase( params.begin() + paramsToDelete[ i ] );
    }

    std::vector<MultiParameterHypothesis> modelHypotheses;

    EXTRAP::MultiParameterTerm mult;

    for ( int i = 0; i < bestSingleParameterHypotheses.size(); i++ )
    {
        EXTRAP::CompoundTerm ct;
        ct = bestSingleParameterHypotheses[ i ].getFunction()->getCompoundTerms()[ 0 ];
        ct.setCoefficient( 1 );
        mult.addCompoundTermParameterPair( ct, params[ i ] );
    }
    std::vector<EXTRAP::MultiParameterTerm> add;
    for ( int i = 0; i < bestSingleParameterHypotheses.size(); i++ )
    {
        EXTRAP::MultiParameterTerm tmp;
        EXTRAP::CompoundTerm       ct;
        ct = bestSingleParameterHypotheses[ i ].getFunction()->getCompoundTerms()[ 0 ];
        ct.setCoefficient( 1 );
        tmp.addCompoundTermParameterPair( ct, params[ i ] );
        add.push_back( tmp );
    }



    EXTRAP::MultiParameterFunction* f1, * f2, * f3, * f4;

    f1 = new  EXTRAP::MultiParameterFunction();
    f2 = new  EXTRAP::MultiParameterFunction();
    f3 = new  EXTRAP::MultiParameterFunction();
    f4 = new  EXTRAP::MultiParameterFunction();
    f1->addMultiParameterTerm( mult );
    f2->addMultiParameterTerm( add[ 0 ] );
    f2->addMultiParameterTerm( mult );
    f3->addMultiParameterTerm( add[ 1 ] );
    f3->addMultiParameterTerm( mult );
    f4->addMultiParameterTerm( add[ 0 ] );
    f4->addMultiParameterTerm( add[ 1 ] );
    MultiParameterHypothesis              mph1( f1 );
    MultiParameterHypothesis              mph2( f2 );
    MultiParameterHypothesis              mph3( f3 );
    MultiParameterHypothesis              mph4( f4 );
    std::vector<MultiParameterHypothesis> hypotheses;
    hypotheses.push_back( mph1 );
    hypotheses.push_back( mph2 );
    hypotheses.push_back( mph3 );
    hypotheses.push_back( mph4 );
    MultiParameterHypothesis bestHypothesis = hypotheses[ 0 ];
    bestHypothesis.estimateParameters( modeledDataPointList );
    bestHypothesis.computeCost( modeledDataPointList );
    bestHypothesis.computeAdjustedRSquared( constantCost, modeledDataPointList );

    for ( int i = 1; i < hypotheses.size(); i++ )
    {
        hypotheses[ i ].estimateParameters( modeledDataPointList );
        hypotheses[ i ].computeCost( modeledDataPointList );
        hypotheses[ i ].computeAdjustedRSquared( constantCost, modeledDataPointList );

        if ( hypotheses[ i ].getSMAPE() < bestHypothesis.getSMAPE() )
        {
            delete ( bestHypothesis.getFunction() );
            bestHypothesis = hypotheses[ i ];
        }
        else
        {
            delete ( hypotheses[ i ].getFunction() );
        }
    }

    return MultiParameterHypothesis( bestHypothesis );
}
}; // Close namespace
