/*
 * This file is part of the Extra-P software (http://www.scalasca.org/software/extra-p)
 *
 * Copyright (c) 2017,
 * Technische Universitaet Darmstadt, Germany
 *
 * This software may be modified and distributed under the terms of
 * a BSD-style license.  See the COPYING file in the package base
 * directory for details.
 *
 */
#include <config.h>
#include <EXTRAP_SingleParameterExhaustiveModelGenerator.hpp>
#include <EXTRAP_SingleParameterExhaustiveFunctionModeler.hpp>

namespace EXTRAP
{
const std::string SingleParameterExhaustiveModelGenerator::SINGLEPARAMETEREXHAUSTIVEMODELGENERATOR_PREFIX = "SingleParameterExhaustiveModelGenerator";

SingleParameterExhaustiveModelGenerator::SingleParameterExhaustiveModelGenerator()
{
    m_modeler = new SingleParameterExhaustiveFunctionModeler();
}

void
SingleParameterExhaustiveModelGenerator::openOutputFile( const std::string& file )
{
    this->m_modeler->m_outputStream.open( file.c_str() );
    assert( this->m_modeler->m_outputStream.is_open() );
    this->m_modeler->writeHeader();
}

void
SingleParameterExhaustiveModelGenerator::closeOutputFile( void )
{
    this->m_modeler->m_outputStream.close();
}

void
SingleParameterExhaustiveModelGenerator::setFunctionName( const std::string& name )
{
    this->m_modeler->m_functionName = name;
}

SingleParameterFunctionModeler&
SingleParameterExhaustiveModelGenerator::getFunctionModeler() const
{
    return *m_modeler;
}

bool
SingleParameterExhaustiveModelGenerator::serialize( IoHelper* ioHelper ) const
{
    SAFE_RETURN( ioHelper->writeString( SINGLEPARAMETEREXHAUSTIVEMODELGENERATOR_PREFIX ) );
    SAFE_RETURN( this->SingleParameterModelGenerator::serialize(  ioHelper ) );
    return true;
}

SingleParameterExhaustiveModelGenerator*
SingleParameterExhaustiveModelGenerator::deserialize( IoHelper* ioHelper )
{
    SingleParameterExhaustiveModelGenerator* generator = new SingleParameterExhaustiveModelGenerator();
    generator->SingleParameterModelGenerator::deserialize( ioHelper );
    return generator;
}

bool
equal( const SingleParameterExhaustiveModelGenerator* lhs,
       const SingleParameterExhaustiveModelGenerator* rhs )
{
    if ( lhs == rhs )
    {
        return true;
    }
    if ( lhs == NULL || rhs == NULL )
    {
        return false;
    }
    bool result = true;
    result &= lhs->getCrossvalidationMethod() == rhs->getCrossvalidationMethod();
    result &= lhs->getEpsilon() == rhs->getEpsilon();
    result &= lhs->getModelOptions() == rhs->getModelOptions();
    return result;
}
}; // Close namespace
