/*
 * This file is part of the Extra-P software (http://www.scalasca.org/software/extra-p)
 *
 * Copyright (c) 2017,
 * Technische Universitaet Darmstadt, Germany
 *
 * This software may be modified and distributed under the terms of
 * a BSD-style license.  See the COPYING file in the package base
 * directory for details.
 *
 */

#include <config.h>
#include <EXTRAP_Utilities.hpp>

//HelperMethod for matrixInverse
static void
subtractMultLineFrom( int minuendLine, int subtrahendLine, double factor, double* matrix, int n )
{
    for ( int i = 0; i < n; i++ )
    {
        matrix[ minuendLine * n + i ] -= ( matrix[ subtrahendLine * n + i ] * factor );
    }
}
//HelperMethod for matrixInverse
static void
swapLines( int i, int j, double* matrix, int n )
{
    double temp;
    for ( int index = 0; index < n; ++index )
    {
        temp = matrix[ i * n + index ];

        matrix[ i * n + index ] = matrix[ j * n + index ];

        matrix[ j * n + index ] = temp;
    }
}
//HelperMethod for matrixInverse
static void
divideLineBy( int line, double divisor, double* matrix, int n )
{
    for ( int i = 0; i < n; ++i )
    {
        matrix[ line * n + i ] = matrix[ line * n + i ] / divisor;
    }
}
//HelperMethod for matrixInverse
static void
makeIdentityMatrix( double* matrix, double* inverse, int n )
{
    for ( int line = 0; line < ( n - 1 ); line++ )
    {
        for ( int i = line + 1; i < n; ++i )
        {
            double factor = matrix[ line * n + i ] / matrix[ i * n + i ];
            subtractMultLineFrom( line, i, factor, matrix, n );
            subtractMultLineFrom( line, i, factor, inverse, n );
        }
    }
}

void
matrixInverse( double* matrix, int n )
{
    //Filling inverse
    double* inverse = new double[ n * n ];
    for ( int i = 0; i < n; ++i )
    {
        for ( int j = 0; j < n; ++j )
        {
            i == j ? inverse[ i * n + j ] = 1 : inverse[ i * n + j ] = 0;
        }
    }
    //5.Do for all Submatrices (first (sub) rows and cols are cutted
    int col = 0;
    for ( int sub = 0; sub < n; sub++ )
    {
        bool found = false;
        //1.Choose the first column that contains at least one non zero value
        for ( int k = sub; k < n; k++ )
        {
            if ( !found )
            {
                for ( int line = sub; line < n; ++line )
                {
                    //1.1Check for non zero values in column
                    if ( matrix[ line * n + k ] != 0 )
                    {
                        found = true;
                        col   = k;
                        if ( line != sub ) //2. if the non zero value is not in the top line swap
                        {
                            swapLines( sub, line, matrix, n );
                            swapLines( sub, line, inverse, n );
                        }
                    }
                }
                //3. Now divide first Line by the first element of the chosen column
                double divisor = matrix[ sub * n + col ];
                divideLineBy( sub, divisor, inverse, n );
                divideLineBy( sub, divisor, matrix, n );
                //4. Substract multiples of first line from the remaining lines( except the first ofc)
                for ( int i = sub + 1; i < n; ++i )
                {
                    double factor = matrix[ i * n + col ] / matrix[ sub * n + col ];
                    subtractMultLineFrom( i, sub, factor, matrix, n );
                    subtractMultLineFrom( i, sub, factor, inverse, n );
                }
            }
        }
    }
    makeIdentityMatrix( matrix, inverse, n );
    //Copy result in right matrix
    for ( int i = 0; i < n; ++i )
    {
        for ( int j = 0; j < n; ++j )
        {
            matrix[ i * n + j ] = inverse[ i * n + j ];
        }
    }

    delete[] inverse;
}
